@@ -1,8 +1,10 @@
+import collections
 import datetime
 import decimal
 import typing
 import uuid
 from enum import Enum
+from pathlib import Path
 
 from readstr.base64 import base64_decode
 
@@ -19,13 +21,18 @@ def reads_generic(type):
 
 
 def reads(func):
-    typehints = typing.get_type_hints(func)
-    returns = typehints['return']
+    if isinstance(func, type):
+        returns = func
+    else:
+        typehints = typing.get_type_hints(func)
+        returns = typehints['return']
     return reads_generic(returns)(func)
 
 
 @reads
-def read_uuid(str_value: str) -> uuid.UUID:
+def read_uuid(str_value: typing.Union[str, uuid.UUID]) -> uuid.UUID:
+    if isinstance(str_value, uuid.UUID):
+        return str_value
     try:
         return uuid.UUID(str_value)
     except ValueError:
@@ -82,6 +89,11 @@ def read_float(str_value: str) -> float:
     return float(str_value)
 
 
+@reads
+def read_complex(str_value: str) -> complex:
+    return complex(str_value)
+
+
 @reads
 def read_decimal(str_value: str) -> decimal.Decimal:
     return decimal.Decimal(str_value)
@@ -99,12 +111,34 @@ def read_date(str_value: str) -> datetime.date:
 
 
 @reads
-def read_tuple(str_value: str, args) -> tuple:
-    return tuple((readstr(arg_value, arg_type) for arg_value, arg_type in zip(str_value.split(','), args)))
+def read_datetime(str_value: str) -> datetime.datetime:
+    if str_value.lower() == 'now':
+        return datetime.datetime.now()
+    return datetime.datetime.fromisoformat(str_value)
 
 
 @reads
-def read_dict(str_value: str, args) -> dict:
+def read_path(str_value: str) -> Path:
+    return Path(str_value)
+
+
+@reads
+def read_tuple(str_value: str, args: tuple) -> tuple:
+    result = tuple((readstr(arg_value, arg_type) for arg_value, arg_type in zip(str_value.split(','), args)))
+    if len(result) != len(args):
+        ValueError(f"not enough values to populate tuple: {str_value} (expected: {len(args)}")
+    return result
+
+
+@reads
+def read_bytes(str_value) -> bytes:
+    return base64_decode(str_value)
+
+
+# noinspection PyUnresolvedReferences
+@reads_generic(collections.abc.Mapping)
+@reads
+def read_dict(str_value: str, args: tuple) -> dict:
     key_type, value_type = args
     result = dict()
     for v in str_value.split(','):
@@ -117,22 +151,26 @@ def read_dict(str_value: str, args) -> dict:
     return result
 
 
+# noinspection PyUnresolvedReferences
+@reads_generic(collections.abc.Sequence)
 @reads
 def read_list(str_value: str, args: tuple) -> list:
-    result = list()
     arg_type, = args
-    for v in str_value.split(','):
-        result.append(readstr(v, arg_type))
-    return result
+    return [readstr(v, arg_type) for v in str_value.split(',')]
+
+
+# noinspection PyUnresolvedReferences
+@reads_generic(collections.abc.Set)
+@reads
+def read_set(str_value: str, args: tuple) -> set:
+    arg_type, = args
+    return {readstr(v, arg_type) for v in str_value.split(',')}
 
 
 @reads
-def read_set(str_value: str, args: tuple) -> set:
-    result = set()
+def read_frozenset(str_value: str, args: tuple) -> frozenset:
     arg_type, = args
-    for v in str_value.split(','):
-        result.add(readstr(v, arg_type))
-    return result
+    return frozenset((readstr(v, arg_type) for v in str_value.split(',')))
 
 
 @reads_generic(typing.Literal)
